﻿(function ($) {
	//	Adds <td> element containing <input> element.
	var appendTd = function (type, name, index, value, event) {
		var td = $("<td></td>");
		var input = $("<input />").attr({
			"type": type,
			"id": name + "-" + index,
			"name": name + "-" + index,
			"class": name
		});
		if (type == "span") {
			input = $("<" + type + "></" + type + ">").attr("class", name).html(value);
		}
		else if (type == "checkbox" || type == "radio") {
			if (value) {
				$(input).attr("checked", "checked");
			}
		}
		else {
			$(input).attr("value", value);
		}
		if (event != undefined) {
			$(input).bind(event.name, event.callback);
		}
		$(td).append(input);
		return td;
	}

	//	Checks if prevalue is empty or not.
	var isPrevalueEmpty = function (key, value) {
		var empty = false;
		if ($(key).val() == "" && $(key).val() == "") {
			empty = true;
		}
		return empty;
	}

	//	Checks if a prevalue key is duplicated or not.
	var isKeyDuplicated = function (prevalue1, prevalue2) {
		var duplicated = false;
		if (prevalue1.key == prevalue2.key) {
			duplicated = true;
		}
		return duplicated;
	}

	//	Adds a prevalue.
	var addPrevalue = function () {
		var prevalues = $("#body_PrevaluesValue").val();
		if (prevalues != undefined && prevalues != "" && prevalues.length > 0) {
			prevalues = JSON.parse(prevalues);
		}
		else {
			prevalues = [];
		}
		if (prevalues.length >= 0) {
			var results = prevalues;
			var add = $(this).parent().parent();
			var key = $(add).find("input#prevalue-key");
			var value = $(add).find("input#prevalue-value");

			if (isPrevalueEmpty(key, value)) {
				alert("All values must be mandatory in proper format");
			}
			else {
				var prevalue = {
					"key": $(key).val(),
					"value": $(value).val()
				};
				var exist = false;
				for (var i = 0; i < results.length; i++) {
					if (isKeyDuplicated(results[i], prevalue)) {
						exist = true;
						break;
					}
				}
				if (exist) {
					alert("Key is duplicated");
				}
				else {
					results.push(prevalue);
					$("#body_PrevaluesValue").val(JSON.stringify(results));

					var index = results.length - 1;

					var tr = $("<tr></tr>").attr("rel", index);
					$(tr).append(appendTd("text", "prevalue-key", index, $(key).val()));
					$(tr).append(appendTd("text", "prevalue-value", index, $(value).val()));
					$(tr).append(appendTd("button", "prevalue-update", index, "Update", { name: "click", callback: updatePrevalue }));
					$(tr).append(appendTd("button", "prevalue-remove", index, "Remove", { name: "click", callback: removePrevalue }));
					$(tr).append(appendTd("span", "ui-sortable-cursor", index, "Sort"));

					$(tr).insertBefore(add);

					$(key).val("");
					$(value).val("");
				} //	if...else
			} //	if...else
		}
	};

	//	Updates a prevalue.
	var updatePrevalue = function () {
		var prevalues = $("#body_PrevaluesValue").val();
		if (prevalues != undefined && prevalues != "" && prevalues.length > 0) {
			prevalues = JSON.parse(prevalues);
		}
		else {
			prevalues = [];
		}
		if (prevalues.length > 0) {
			var results = prevalues;
			var update = $(this).parent().parent();
			var key = $(update).find("input[id^='prevalue-key-']");
			var value = $(update).find("input[id^='prevalue-value-']");

			if (isPrevalueEmpty(key, value)) {
				alert("All values must be mandatory in proper format");
			}
			else {
				var prevalue = {
					"key": $(key).val(),
					"value": $(value).val()
				};
				var exist = false;
				for (var i = 0; i < results.length; i++) {
					if (isKeyDuplicated(results[i], prevalue)) {
						exist = true;
						break;
					}
				}
				if (exist) {
					alert("Key is duplicated");
				}
				else {
					var index = parseInt($(update).attr("rel"));
					results.splice(index, 1, prevalue);
					$("#body_PrevaluesValue").val(JSON.stringify(results));
				} //	if...else
			} //	if...else
		}
	};

	//	Removes an prevalue.
	var removePrevalue = function () {
		var prevalues = $("#body_PrevaluesValue").val();
		if (prevalues != undefined && prevalues != "" && prevalues.length > 0) {
			prevalues = JSON.parse(prevalues);
		}
		else {
			prevalues = [];
		}
		if (prevalues.length > 0) {
			var results = prevalues;
			var table = $(this).parent().parent().parent();
			var remove = $(this).parent().parent();

			var index = parseInt($(remove).attr("rel"));
			results.splice(index, 1);
			$("#body_PrevaluesValue").val(JSON.stringify(results));

			var trs = $(remove).parent().find("tr[rel][rel!='-1']");
			$(trs).remove();

			$.each(results, function (i, prevalue) {
				var tr = $("<tr></tr>").attr("rel", i);
				$(tr).append(appendTd("text", "prevalue-key", i, prevalue.key));
				$(tr).append(appendTd("text", "prevalue-value", i, prevalue.value));
				$(tr).append(appendTd("button", "prevalue-update", i, "Update", { name: "click", callback: updatePrevalue }));
				$(tr).append(appendTd("button", "prevalue-remove", i, "Remove", { name: "click", callback: removePrevalue }));
				$(tr).append(appendTd("span", "ui-sortable-cursor", i, "Sort"));

				$(tr).insertBefore($(table).find("tr:last"));
			}); //	$.each
		}
	};

	$(document).ready(function () {
		var table = $("table#prevalues-list");

		//	Sets the sortable event on each <tr>.
		$(table).sortable({
			items: "tr:not(.not-sortable)",
			placeholder: "ui-state-highlight",
			update: function (event, ui) {
				var trs = $(table).find("tr[rel][rel!='-1']");
				var prevalues = [];
				$.each(trs, function (i, tr) {
					var key = $(tr).find("input[id^='prevalue-key']");
					var value = $(tr).find("input[id^='prevalue-value']");
					var prevalue = {
						"key": $(key).val(),
						"value": $(value).val()
					};
					prevalues.push(prevalue);
					$("#body_PrevaluesValue").val(JSON.stringify(prevalues));
				}); //	$.each
			} //	$.change
		}); //	$.sortable

		//	Sets the click events on each <tr>.
		$(table).find("tr").filter("[rel]").each(function (i) {
			$(this).find("input[id^='prevalue-add']").click(addPrevalue);
			$(this).find("input[id^='prevalue-update-']").click(updatePrevalue);
			$(this).find("input[id^='prevalue-remove-']").click(removePrevalue);
		}); //	$.each
	}); //	$.ready
})(jQuery);
